#!/bin/ksh
#\
exec tclsh "$0" "$@"

proc read_user_settings {fname} {
global labelbgcolor labelfgcolor labelfont 
global entrybgcolor entryfgcolor entryfont
global buttonbgcolor buttonfgcolor buttonfont

if [file exists $fname] {
	set fid [open $fname RDONLY]
	while {![eof $fid]} {
		gets $fid buf
		if {[string index $buf 0]=="#"} continue
		set buflist [split "$buf" ":"]
		if ![llength $buflist] continue
		set var [lindex $buflist 0]
		if {[string trim $var]=={}} continue
		set value [string trim [lindex $buflist 1]]
		if {[string trim $value]=={}} continue
		global $var
		set $var $value
	}
}
}

proc read_from_rex {} {
global localzonelist timezonelist syncrangelist todolist memoscatlist
global contactscatlist contactlist
global prefslist listlist fieldlabels
global ncontacts nmemos
global rexdir rexfile proconnect pid tmpdir
set ncontacts -1
set nmemos -1

catch {exec $proconnect [lindex $rexfile 0] [lindex $rexfile 1] -o $tmpdir/.proconnect$pid.out} {msg}
if {$msg!={}} {
	puts "Failed! ($msg - device [lindex $rexfile 1])"
	exit 1
}
set fname $tmpdir/.proconnect$pid.out

if [file exists $fname] {
	set fid [open $fname RDONLY]
	while {![eof $fid]} {
		gets $fid buf
		if {[string index $buf 0]=="#"} continue
		switch -regexp $buf {
			^reset {}
			^localzone {
					set curlist localzonelist
					lappend $curlist "$buf"
			}
			^syncrange {
					set curlist syncrangelist
					lappend $curlist "$buf"
			}
			^todo {
					set curlist todolist
					lappend $curlist "$buf"
			}
			^timezone {
					set curlist timezonelist
					lappend $curlist "$buf"
			  }
			^prefs	{
					set curlist prefslist
					lappend $curlist "$buf"
			}
			^memoscategories {
					set curlist memoscatlist
					lappend $curlist "$buf"
				}
			^memo {
					incr nmemos
					global memolist$nmemos
					set curlist memolist$nmemos
					lappend $curlist "$buf"
			}
			^contactscategories {
					set curlist contactscatlist
					lappend $curlist "$buf"
			}
			^fieldlabels {
					set curlist fieldlist
					lappend $curlist "$buf"
			}
			^contact {
					incr ncontacts
					global contactlist$ncontacts
					set curlist contactlist$ncontacts
					lappend $curlist "$buf"
			}
			^list {
					set curlist listlist
					lappend $curlist "$buf"
			}
			^done {
					break
			}
			default {
				if {$buf != {}} {
					lappend $curlist "$buf"
				}
			}
		}
	}
close $fid
}
if [info exists l] {unset l}

# Merge the fieldlabels
for {set i 1} {$i <= [llength $fieldlist]} {incr i} {
	set label "[lindex [lindex $fieldlist $i] 1]"
	if {$label!={}} {
		set fieldlabels [lreplace $fieldlabels $i $i $label]
	}
}

# Massage contacts categories
for {set i 1} {$i <= [llength $contactscatlist]} {incr i} {
	set j [lindex $contactscatlist $i]
	if {$j!={}} {
		lappend l [join $j]
	}
}
set contactscatlist $l
if [info exists l] {unset l}

# Massage memo categories
for {set i 1} {$i <= [llength $memoscatlist]} {incr i} {
	set j [lindex $memoscatlist $i]
	if {$j!={}} {
		lappend l [join $j]
	}
}
set memoscatlist $l

# Massage the memo lists
for {set i 0} {$i <= $nmemos} {incr i} {
	global memolist$i
	set l [set memolist$i]
	set n [llength $l]
	
	for {set x 0} {$x < $n} {incr x} {
		set j [lindex $l $x]
		set h [string last # $j]
		if {$h>=0} {
			set h [expr $h -1]
			set l [lreplace $l $x $x "[string range $j 0 $h]"]
		}
	}
	set memolist$i [join $l \n]
}
}

proc instext {w text} {
	$w configure -state normal
	$w delete 0 end
	$w insert 0 "$text"
	$w configure -state disabled
}

proc save_user_settings {var val} {
	global configfile
	set done 0
	set fod [open $configfile.bak {WRONLY TRUNC CREAT}]

	if [file exists $configfile] {
		set fid [open $configfile RDONLY]
		while {![eof $fid]} {
			gets $fid buf
			set buflist [split "$buf" ":"]
			if ![llength $buflist] continue
			set value [string trim [lindex $buflist 1]]
			if [string match [lindex $buflist 0] $var] {
				puts $fod "$var: $val"
				incr done
				continue
			}
			puts $fod $buf
		}
	}
	if !$done {puts $fod "$var: $val"}
	close $fod
	close $fid
	catch {
		file copy -force $configfile.bak $configfile
		file delete -force $configfile.bak
	}
}

proc convstr {str} {
set ret {}
for {set i 0} {$i < [string length $str]} {incr i} {
	set c [string index $str $i]
	if {$c=="\""} {append ret "\\"}
	append ret $c
}
return $ret
}

proc _iframe {w text args} {
array set varargs $args
global labelbgcolor labelfgcolor labelfont entrybgcolor entryfgcolor entryfont
	frame $w -bg $labelbgcolor
	frame $w.f -re groove -bd 2 -bg $labelbgcolor
	label $w.l -text $text \
		-bg $labelbgcolor -fg $labelfgcolor -font $labelfont \
		-justify left -width 12
#	place $w.f -in $w -relwidth .9 -relheight .9 -relx .1 -rely .05
#	place $w.l -in $w
	pack $w.l -side left
	pack $w.f -side right -expand yes -fill x
	foreach opts [array names varargs] {$w config $opts $varargs($opts)}
}

proc _frame {w args} {
array set varargs $args
global labelbgcolor labelfgcolor labelfont
	frame $w -bg $labelbgcolor
	frame $w.f -re groove -bd 2 -bg $labelbgcolor
	place $w.f -in $w -relwidth .9 -relheight .9 -relx .05 -rely .05
	foreach opts [array names varargs] {$w config $opts $varargs($opts)}
}

proc _save_all {} {
global localzonelist timezonelist syncrangelist todolist memoscatlist
global contactscatlist contactlist
global prefslist listlist
global ncontacts nmemos fieldlabels
global rexdir rexfile proconnect pid tmpdir

set fname $tmpdir/.proconnect$pid.in
set fid [open $fname {WRONLY CREAT TRUNC}]

	puts $fid "reset"

	foreach list [list timezonelist localzonelist syncrangelist \
		     todolist prefslist listlist] {
		if [info exists $list] {
			foreach i [set $list] {
				puts $fid $i
			}
		}
	}

	if [info exists contactscatlist] {
		puts $fid "contactscategories"
		foreach i $contactscatlist {puts $fid "\t\"$i\""}
	}

	if [info exists memoscatlist] {
		puts $fid "memoscategories"
		foreach i $memoscatlist {puts $fid "\t\"$i\""}
	}

	if [info exists fieldlabels] {
		puts $fid "fieldlabels"
		for {set i 1} {$i<[llength $fieldlabels]} {incr i} {
			puts $fid "\t0 \"[lindex $fieldlabels $i]\""
		}
	}

	for {set i 0} {$i <=$ncontacts} {incr i} {
		global contactlist$i
		if ![info exists contactlist$i] continue
		set j 0
		foreach x [set contactlist$i] {
			if !$j {
				puts $fid $x
			} else {
				puts $fid "\t[lindex $x 0] [lindex $x 1] \"[lindex $x 2]\""
			}
			incr j
		}
	}

	for {set i 0} {$i <=$nmemos} {incr i} {
		global memolist$i
		if ![info exists memolist$i] continue
		puts $fid "[set memolist$i]"
	}


close $fid

catch {exec $proconnect [lindex $rexfile 0] [lindex $rexfile 1] -i $tmpdir/.proconnect$pid.in} {msg}
return $msg
}

#
#For debuggering
#
#set tmpdir .
#set rexdir .
#set proconnect ./ProConnect
#set rexfile rexdata
#set pid 0
#read_from_rex
#_save_all
